#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Path = require("path");
var mkdirp_1 = require("mkdirp");
var fs = require("fs");
var child_process_1 = require("child_process");
var assert = require("assert");
var constants_1 = require("./constants");
var util_1 = require("./util");
var log = console.log.bind(console);
function main() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        function extractAppImage() {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var filesToExtract;
                return tslib_1.__generator(this, function (_a) {
                    mkdirp_1.sync(extractionTargetPath, { mode: 448 });
                    // Set the execute bit
                    fs.chmodSync(appImageDownloadPath, fs.statSync(appImageDownloadPath).mode | 64);
                    filesToExtract = constants_1.filesInAppImage;
                    filesToExtract.forEach(function (fileToExtract, i) {
                        log("Extracting " + fileToExtract + " (" + Math.round(i / filesToExtract.length * 100) + "%)...");
                        var spawnResult = child_process_1.spawnSync(appImageDownloadPath, ['--appimage-extract', fileToExtract], {
                            cwd: extractionTargetPath,
                            encoding: 'utf8'
                        });
                        var stdout = spawnResult.stdout, stderr = spawnResult.stderr;
                        if (spawnResult.status) {
                            throw new Error('AppImage failed to extract:\n' + stderr);
                        }
                        // catch the expected inode_type 8 failures
                        var expected = 'TODO: Implement inode.base.inode_type 8';
                        if (spawnResult.stderr.indexOf(expected) > -1) {
                            var dir = Path.join(extractionTargetPath, 'squashfs-root', fileToExtract);
                            log("Manually creating directory: " + dir);
                            mkdirp_1.sync(dir, { mode: 448 });
                        }
                    });
                    return [2 /*return*/];
                });
            });
        }
        var ownBinPath, foundPath, _i, _a, stat, executeBits, cacheInstallationDirectory, symlinkTarget, cacheManifestPath, manifest, appImageDownloadPath, extractionTargetPath, _b, _c, entryPointPath, alternativeEntryPointPath;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    // Skip Windows entirely; it already has PowerShell.
                    if (process.platform === 'win32') {
                        console.log('Windows comes with Powershell preinstalled; no action required.');
                        process.exit(0);
                    }
                    ownBinPath = Path.normalize(Path.resolve(Path.join(require.resolve('../package.json'), '../../.bin/powershell')));
                    found: {
                        for (_i = 0, _a = process.env.PATH.split(':').map(function (v) { return Path.join(v, 'powershell'); }); _i < _a.length; _i++) {
                            foundPath = _a[_i];
                            // Skip the powershell command that this package puts on the path; that's not the one we want
                            if (Path.normalize(foundPath) === ownBinPath)
                                continue;
                            try {
                                stat = fs.statSync(foundPath);
                                executeBits = stat.mode & 73;
                                if (executeBits)
                                    break found;
                            }
                            catch (e) { }
                        }
                        // did not find it
                        foundPath = null;
                    }
                    if (foundPath) {
                        log('Found powershell on PATH; no action required.');
                        // process.exit(0);
                    }
                    // TODO add support for Mac
                    if (process.platform === 'darwin') {
                        log('macos is not yet supported.');
                        process.exit(1);
                    }
                    cacheInstallationDirectory = Path.resolve(Path.join(process.env.npm_config_prefix, 'node_modules/@cspotcode/get-powershell--cache'));
                    mkdirp_1.sync(cacheInstallationDirectory, { mode: 448 });
                    cacheManifestPath = Path.join(cacheInstallationDirectory, 'cache-manifest.json');
                    manifest = util_1.readJsonFileWithDefault(cacheManifestPath, {});
                    if (manifest[constants_1.nameAndVersion]) {
                        symlinkTarget = manifest[constants_1.nameAndVersion];
                        try {
                            assert(fs.statSync(symlinkTarget).mode & 64);
                        }
                        catch (e) {
                            symlinkTarget = undefined;
                        }
                    }
                    appImageDownloadPath = Path.join(cacheInstallationDirectory, constants_1.nameAndVersion + ".AppImage");
                    extractionTargetPath = Path.join(cacheInstallationDirectory, "" + constants_1.nameAndVersion);
                    if (!!symlinkTarget) return [3 /*break*/, 8];
                    _b = fs.existsSync(appImageDownloadPath);
                    if (!_b) return [3 /*break*/, 2];
                    return [4 /*yield*/, util_1.sha256OfFile(appImageDownloadPath)];
                case 1:
                    _b = (_d.sent()) === constants_1.appImageSha256;
                    _d.label = 2;
                case 2:
                    if (!_b) return [3 /*break*/, 3];
                    log("Found AppImage on disk; skipping download. (" + appImageDownloadPath + ")");
                    return [3 /*break*/, 6];
                case 3:
                    // Download the AppImage archive
                    log("Downloading Powershell AppImage from " + constants_1.appImageUrl + " to " + appImageDownloadPath + "...");
                    return [4 /*yield*/, util_1.downloadUrlToFile(constants_1.appImageUrl, appImageDownloadPath)];
                case 4:
                    _d.sent();
                    log('Download finished.');
                    _c = assert;
                    return [4 /*yield*/, util_1.sha256OfFile(appImageDownloadPath)];
                case 5:
                    _c.apply(void 0, [(_d.sent()) === constants_1.appImageSha256, 'SHA256 verification failed; download appears corrupt.']);
                    _d.label = 6;
                case 6:
                    // Extract the entire AppImage.  Not all systems support FUSE; extracting the AppImage eliminates that requirement.
                    log("Extracting AppImage to " + extractionTargetPath + "...");
                    return [4 /*yield*/, extractAppImage()];
                case 7:
                    _d.sent();
                    log("Extracted to " + extractionTargetPath);
                    // Create an alternative entry-point that bypasses the `.wrapper` script
                    log("Creating entry-point that bypasses AppImage's .wrapper script.");
                    entryPointPath = Path.join(extractionTargetPath, 'squashfs-root/AppRun');
                    alternativeEntryPointPath = Path.join(extractionTargetPath, 'squashfs-root/AppRun-skip-wrapper');
                    fs.writeFileSync(alternativeEntryPointPath, fs.readFileSync(entryPointPath, 'utf8').replace('.wrapper', ''));
                    fs.chmodSync(alternativeEntryPointPath, 448);
                    symlinkTarget = alternativeEntryPointPath;
                    // Store the desired symlink target into our cache manifest
                    util_1.patchJsonFile(cacheManifestPath, {
                        indent: true, defaultValue: {}
                    }, function (json) {
                        json[constants_1.nameAndVersion] = symlinkTarget;
                    });
                    _d.label = 8;
                case 8:
                    log("Symlinking from node_modules/.bin/powershell to " + symlinkTarget + "...");
                    mkdirp_1.sync(Path.dirname(ownBinPath), { mode: 448 });
                    fs.symlinkSync(symlinkTarget, ownBinPath);
                    log("Done!");
                    return [2 /*return*/];
            }
        });
    });
}
main().catch(function (err) {
    console.error(err);
    process.exit(1);
});
// First download and extract the appimage tool
// https://github.com/AppImage/AppImageKit/releases/download/continuous/appimagetool-x86_64.AppImage
// Use it to list all files in the powershell appimage
//  * invoke with -l <path to powershell appimage>
//  * discard lines beginning with WARNING:
// manually extract powershell appimage files one at a time
//  * while looping, if the file thrown an error about inode 8, instead mkdir -p and manually set 700 permissions
